import random

from . import settings
from .utils import load, load_cached, update


class UserAgent(object):
    def __init__(self, cache=True):
        if cache:
            self.data = load_cached()
        else:
            self.data = load()

        self.cache = cache

    def update(self, cache=None):
        if cache is None:
            cache = self.cache

        if self.cache:
            update()

        self.__init__(cache=cache)

    def __getitem__(self, attr):
        return self.__getattr__(attr)

    def __getattr__(self, attr):
        for replacement in settings.REPLACEMENTS:
            attr = attr.replace(replacement, '')

        attr = attr.lower()

        if attr == 'random':
            attr = self.data['randomize'][
                str(random.randint(0, len(self.data['randomize']) - 1))
            ]
        else:
            for shortcut, value in settings.SHORTCUTS:
                if attr == shortcut:
                    attr = value
                    break

        try:
            return self.data['browsers'][attr][
                random.randint(
                    0, len(self.data['browsers'][attr]) - 1
                )
            ]
        except KeyError:
            return None



import os
import tempfile

DB = os.path.join(
    tempfile.gettempdir(), 'fake_useragent.json'
)

BROWSERS_STATS_PAGE = 'http://www.w3schools.com/browsers/browsers_stats.asp'

BROWSER_BASE_PAGE = 'http://useragentstring.com/pages/%s/'

BROWSERS_COUNT_LIMIT = 30

REPLACEMENTS = (' ', '_')

SHORTCUTS = (
    ('internet explorer', 'internetexplorer'),
    ('ie', 'internetexplorer'),
    ('msie', 'internetexplorer'),
    ('google', 'chrome'),
    ('googlechrome', 'chrome'),
    ('ff', 'firefox')
)

OVERRIDES = (
    ('Internet Explorer', 'IE'),
)



import os
import re

from . import settings

try:  # Python 2
    from urllib import urlopen, quote_plus
except ImportError:  # Python 3
    from urllib.request import urlopen
    from urllib.parse import quote_plus
try:
    import json
except ImportError:
    import simplejson as json


def get(url, annex=None):
    if annex is not None:
        url = url % (quote_plus(annex), )
    return urlopen(url).read()


def get_browsers():
    """
    very very hardcoded/dirty re/split stuff, but no dependencies
    """
    html = get(settings.BROWSERS_STATS_PAGE)
    html = html.decode('windows-1252')
    html = html.split('<table class="w3-table-all notranslate">')[1]
    html = html.split('</table>')[0]

    browsers = re.findall(r'\.asp">(.+?)<', html, re.UNICODE)

    for value, override in settings.OVERRIDES:
        browsers = [
            value if browser == override else browser
            for browser in browsers
        ]

    browsers_statistics = re.findall(
        r'td\sclass="right">(.+?)\s', html, re.UNICODE
    )

    # TODO: ensure encoding

    return list(zip(browsers, browsers_statistics))


def get_browser_versions(browser):
    """
    very very hardcoded/dirty re/split stuff, but no dependencies
    """
    html = get(settings.BROWSER_BASE_PAGE, browser)
    html = html.decode('iso-8859-1')
    html = html.split('<div id=\'liste\'>')[1]
    html = html.split('</div>')[0]

    browsers_iter = re.finditer(r'\.php\'>(.+?)</a', html, re.UNICODE)

    count = 0

    browsers = []

    for browser in browsers_iter:
        if 'more' in browser.group(1).lower():
            continue

        # TODO: ensure encoding
        browsers.append(browser.group(1))
        count += 1

        if count == settings.BROWSERS_COUNT_LIMIT:
            break

    return browsers


def load():
    browsers_dict = {}
    randomize_dict = {}

    for item in get_browsers():
        browser, percent = item

        browser_key = browser

        for replacement in settings.REPLACEMENTS:
            browser_key = browser_key.replace(replacement, '')

        browser_key = browser_key.lower()

        browsers_dict[browser_key] = get_browser_versions(browser)

        for counter in range(int(float(percent))):
            randomize_dict[str(len(randomize_dict))] = browser_key

    db = {}
    db['browsers'] = browsers_dict
    db['randomize'] = randomize_dict

    return db


def write(data):
    data = json.dumps(data, ensure_ascii=False)

    # no codecs\with for python 2.5
    f = open(settings.DB, 'w+')
    f.write(data)
    f.close()


def read():
    # no codecs\with for python 2.5
    f = open(settings.DB, 'r')
    data = f.read()
    f.close()

    return json.loads(data)


def exist():
    return os.path.isfile(settings.DB)


def rm():
    if exist():
        os.remove(settings.DB)


def update():
    if exist():
        rm()

    write(load())


def load_cached():
    if not exist():
        update()

    return read()



import sys
from setuptools import setup


classifiers = '''\
Environment :: Web Environment
Intended Audience :: Developers
Topic :: Internet :: WWW/HTTP
Topic :: Software Development :: Libraries
License :: OSI Approved :: Apache Software License
Development Status :: 5 - Production/Stable
Natural Language :: English
Programming Language :: Python
Programming Language :: Python :: 2
Programming Language :: Python :: 3
Operating System :: OS Independent
'''

description = 'Up to date simple useragent faker with real world database'

packages = ['fake_useragent']

install_requires = []


# no codecs\with for python 2.5
def long_description():
    f = open('README.rst')
    rst = f.read()
    f.close()
    return rst


# Python 2.5 capability
major, minor = sys.version_info[:2]
if (major, minor) == (2, 5):
    install_requires.append('simplejson')


setup(
    name='fake-useragent',
    version='0.0.8',
    packages=packages,
    description=description,
    # include_package_data=True,
    long_description=long_description(),
    install_requires=install_requires,
    author='hellysmile',
    author_email='hellysmile@gmail.com',
    url='https://github.com/hellysmile/fake-useragent',
    zip_safe=False,
    license='http://www.apache.org/licenses/LICENSE-2.0',
    classifiers=filter(None, classifiers.split('\n')),
    keywords=[
        'user', 'agent', 'user agent', 'user-agent', 'fake'
    ]
)
